<?php

namespace FiloBlu\AdminSecurity\Observer;

use FiloBlu\AdminSecurity\Helper\Data;
use FiloBlu\AdminSecurity\Model\Ip;
use FiloBlu\GcpLogging\Logger\Logger;
use Magento\Backend\Model\Auth\Session;
use Magento\Framework\Event\Observer;
use Magento\Framework\Event\ObserverInterface;

class LoginFailedObserver implements ObserverInterface
{
    /**
     * @var Session
     */
    protected $authStorage;

    /**
     * @var Logger
     */
    protected $gcpLogger;

    /**
     * @var Data
     */
    protected $helper;

    /**
     * @var Ip
     */
    protected $ipModel;

    /**
     * @param Session $authStorage
     * @param Logger $gcpLogger
     */
    public function __construct(
        Session $authStorage,
        Logger  $gcpLogger,
        Data    $helper,
        Ip      $ipModel
    )
    {
        $this->authStorage = $authStorage;
        $this->gcpLogger = $gcpLogger;
        $this->helper = $helper;
        $this->ipModel = $ipModel;
    }

    /**
     * Check Login Backend
     *
     * @param Observer $observer
     * @throws AuthenticationException
     */
    public function execute(Observer $observer)
    {
        if (!$this->authStorage->getWaitingOtpVerification()) {
            $logData = $this->helper->createLogData($observer->getEvent()->getUserName(), null, $this->ipModel->getCurrentIp());
            $this->gcpLogger->warning('Login failed', $logData);
        }
    }
}
