<?php

namespace FiloBlu\AdminSecurity\Ui\Component\Form\User;

use FiloBlu\AdminSecurity\Api\TrustedRepositoryInterface;
use FiloBlu\AdminSecurity\Model\Trusted;
use Magento\Framework\Api\SearchCriteriaBuilder;
use Magento\Framework\UrlInterface;
use Magento\Ui\DataProvider\AbstractDataProvider;
use Magento\User\Model\ResourceModel\User\CollectionFactory;
use Magento\User\Model\User;

class DataProvider extends AbstractDataProvider
{
    private $loadedData = null;

    /**
     * @var TrustedRepositoryInterface
     */
    private $trustedRepository;

    /**
     * @var UrlInterface
     */
    private $url;

    /**
     * @var SearchCriteriaBuilder
     */
    private $searchCriteriaBuilder;

    /**
     * DataProvider constructor.
     * @param CollectionFactory $collectionFactory
     * @param UrlInterface $url
     * @param TrustedRepositoryInterface $trustedManager
     * @param SearchCriteriaBuilder $searchCriteriaBuilder
     * @param string $name
     * @param string $primaryFieldName
     * @param string $requestFieldName
     * @param array $meta
     * @param array $data
     * @SuppressWarnings("PHPMD.ExcessiveParameterList")
     */
    public function __construct(
        CollectionFactory $collectionFactory,
        UrlInterface $url,
        TrustedRepositoryInterface $trustedRepository,
        SearchCriteriaBuilder $searchCriteriaBuilder,
        $name,
        $primaryFieldName,
        $requestFieldName,
        array $meta = [],
        array $data = []
    ) {
        parent::__construct($name, $primaryFieldName, $requestFieldName, $meta, $data);
        $this->collection = $collectionFactory->create();
        $this->trustedRepository = $trustedRepository;
        $this->url = $url;
        $this->searchCriteriaBuilder = $searchCriteriaBuilder;
    }

    /**
     * Get a list of trusted devices as array
     * @param User $user
     * @return array
     */
    private function getTrustedDevices(User $user)
    {
        $trustedDevices = $this->getDevices($user->getId());
        $res = [];

        foreach ($trustedDevices as $trustedDevice) {
            /** @var Trusted $trustedDevice */
            $revokeUrl = $this->url->getUrl('filobluas/tfa/revoke', [
                'id' => $trustedDevice->getId(),
                'user_id' => $user->getId(),
            ]);

            $res[] = [
                'last_ip' => $trustedDevice->getLastIp(),
                'date_time' => $trustedDevice->getDateTime(),
                'device_name' => $trustedDevice->getDeviceName(),
                'revoke_url' => $revokeUrl,
            ];
        }

        return $res;
    }


    /**
     * @inheritdoc
     */
    public function getData()
    {
        if ($this->loadedData === null) {
            $this->loadedData = [];
            $items = $this->collection->getItems();

            /** @var User $user */
            foreach ($items as $user) {
                $trustedDevices = $this->getTrustedDevices($user);

                $data = [
                    'trusted_devices' => $trustedDevices,
                ];
                $this->loadedData[$user->getId()] = $data;
            }
        }

        return $this->loadedData;
    }

    /**
     * Return a list of trusted devices for given user id
     * @param $userId
     * @return \FiloBlu\AdminSecurity\Api\Data\TrustedInterface[]
     */
    private function getDevices($userId)
    {
        $criteria = $this->searchCriteriaBuilder->addFilter('user_id', $userId)->create();
        $results = $this->trustedRepository->getList($criteria);
        
        return $results->getItems();
    }
}
